@@ -29,6 +29,7 @@ class SparseBatchIteratorImpl : public BatchIteratorImpl {
   explicit SparseBatchIteratorImpl(SparsePageSource* source) : source_(source) {
     CHECK(source_ != nullptr);
   }
+  SparsePage& operator*() override { return source_->Value(); }
   const SparsePage& operator*() const override { return source_->Value(); }
   void operator++() override { at_end_ = !source_->Next(); }
   bool AtEnd() const override { return at_end_; }
@@ -81,12 +82,12 @@ float SparsePageDMatrix::GetColDensity(size_t cidx) {
   if (col_density_.empty()) {
     std::vector<size_t> column_size(this->Info().num_col_);
     for (const auto &batch : this->GetColumnBatches()) {
-      for (int i = 0; i < batch.Size(); i++) {
+      for (auto i = 0u; i < batch.Size(); i++) {
         column_size[i] += batch[i].size();
       }
     }
     col_density_.resize(column_size.size());
-    for (int i = 0; i < col_density_.size(); i++) {
+    for (auto i = 0u; i < col_density_.size(); i++) {
       size_t nmiss = this->Info().num_row_ - column_size[i];
       col_density_[i] =
           1.0f - (static_cast<float>(nmiss)) / this->Info().num_row_;
