@@ -38,7 +38,7 @@ DMLC_REGISTRY_LINK_TAG(multiclass_obj_gpu);
 DMLC_REGISTRY_LINK_TAG(regression_obj);
 DMLC_REGISTRY_LINK_TAG(hinge_obj);
 DMLC_REGISTRY_LINK_TAG(multiclass_obj);
-#endif
+#endif  // XGBOOST_USE_CUDA
 DMLC_REGISTRY_LINK_TAG(rank_obj);
 }  // namespace obj
 }  // namespace xgboost
