@@ -19,7 +19,6 @@ namespace tree {
 DMLC_REGISTRY_FILE_TAG(updater_refresh);
 
 /*! \brief pruner that prunes a tree after growing finishs */
-template<typename TStats>
 class TreeRefresher: public TreeUpdater {
  public:
   void Init(const std::vector<std::pair<std::string, std::string> >& args) override {
@@ -31,14 +30,13 @@ class TreeRefresher: public TreeUpdater {
               const std::vector<RegTree*> &trees) override {
     if (trees.size() == 0) return;
     const std::vector<GradientPair> &gpair_h = gpair->ConstHostVector();
-    // number of threads
     // thread temporal space
-    std::vector<std::vector<TStats> > stemp;
+    std::vector<std::vector<GradStats> > stemp;
     std::vector<RegTree::FVec> fvec_temp;
     // setup temp space for each thread
     const int nthread = omp_get_max_threads();
     fvec_temp.resize(nthread, RegTree::FVec());
-    stemp.resize(nthread, std::vector<TStats>());
+    stemp.resize(nthread, std::vector<GradStats>());
     #pragma omp parallel
     {
       int tid = omp_get_thread_num();
@@ -46,16 +44,13 @@ class TreeRefresher: public TreeUpdater {
       for (auto tree : trees) {
         num_nodes += tree->param.num_nodes;
       }
-      stemp[tid].resize(num_nodes, TStats(param_));
-      std::fill(stemp[tid].begin(), stemp[tid].end(), TStats(param_));
+      stemp[tid].resize(num_nodes, GradStats());
+      std::fill(stemp[tid].begin(), stemp[tid].end(), GradStats());
       fvec_temp[tid].Init(trees[0]->param.num_feature);
     }
     // if it is C++11, use lazy evaluation for Allreduce,
     // to gain speedup in recovery
-#if __cplusplus >= 201103L
-    auto lazy_get_stats = [&]()
-#endif
-    {
+    auto lazy_get_stats = [&]() {
       const MetaInfo &info = p_fmat->Info();
       // start accumulating statistics
       for (const auto &batch : p_fmat->GetRowBatches()) {
@@ -86,11 +81,7 @@ class TreeRefresher: public TreeUpdater {
         }
       }
     };
-#if __cplusplus >= 201103L
     reducer_.Allreduce(dmlc::BeginPtr(stemp[0]), stemp[0].size(), lazy_get_stats);
-#else
-    reducer_.Allreduce(dmlc::BeginPtr(stemp[0]), stemp[0].size());
-#endif
     // rescale learning rate according to size of trees
     float lr = param_.learning_rate;
     param_.learning_rate = lr / trees.size();
@@ -111,21 +102,22 @@ class TreeRefresher: public TreeUpdater {
                               const std::vector<GradientPair> &gpair,
                               const MetaInfo &info,
                               const bst_uint ridx,
-                              TStats *gstats) {
+                              GradStats *gstats) {
     // start from groups that belongs to current data
     auto pid = static_cast<int>(info.GetRoot(ridx));
-    gstats[pid].Add(gpair, info, ridx);
+    gstats[pid].Add(gpair[ridx]);
     // tranverse tree
     while (!tree[pid].IsLeaf()) {
       unsigned split_index = tree[pid].SplitIndex();
       pid = tree.GetNext(pid, feat.Fvalue(split_index), feat.IsMissing(split_index));
-      gstats[pid].Add(gpair, info, ridx);
+      gstats[pid].Add(gpair[ridx]);
     }
   }
-  inline void Refresh(const TStats *gstats,
+  inline void Refresh(const GradStats *gstats,
                       int nid, RegTree *p_tree) {
     RegTree &tree = *p_tree;
-    tree.Stat(nid).base_weight = static_cast<bst_float>(gstats[nid].CalcWeight(param_));
+    tree.Stat(nid).base_weight =
+        static_cast<bst_float>(CalcWeight(param_, gstats[nid]));
     tree.Stat(nid).sum_hess = static_cast<bst_float>(gstats[nid].sum_hess);
     if (tree[nid].IsLeaf()) {
       if (param_.refresh_leaf) {
@@ -133,9 +125,9 @@ class TreeRefresher: public TreeUpdater {
       }
     } else {
       tree.Stat(nid).loss_chg = static_cast<bst_float>(
-          gstats[tree[nid].LeftChild()].CalcGain(param_) +
-          gstats[tree[nid].RightChild()].CalcGain(param_) -
-          gstats[nid].CalcGain(param_));
+          xgboost::tree::CalcGain(param_, gstats[tree[nid].LeftChild()]) +
+          xgboost::tree::CalcGain(param_, gstats[tree[nid].RightChild()]) -
+          xgboost::tree::CalcGain(param_, gstats[nid]));
       this->Refresh(gstats, tree[nid].LeftChild(), p_tree);
       this->Refresh(gstats, tree[nid].RightChild(), p_tree);
     }
@@ -143,13 +135,13 @@ class TreeRefresher: public TreeUpdater {
   // training parameter
   TrainParam param_;
   // reducer
-  rabit::Reducer<TStats, TStats::Reduce> reducer_;
+  rabit::Reducer<GradStats, GradStats::Reduce> reducer_;
 };
 
 XGBOOST_REGISTER_TREE_UPDATER(TreeRefresher, "refresh")
 .describe("Refresher that refreshes the weight and statistics according to data.")
 .set_body([]() {
-    return new TreeRefresher<GradStats>();
+    return new TreeRefresher();
   });
 }  // namespace tree
 }  // namespace xgboost
